==> cruft_0.9.6-0.4.tar.gz: cruft-0.9.6/filter_shell.c <==
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define FALSE 0
#define TRUE  1

/* 0 on no match, non-zero on match */
int shellexp( char* string, char* pattern ) {
    /*  printf( "...matching( \"%s\", \"%s\" )\n", string, pattern ); */
    
    switch( pattern[0] ) {
    case '\0':
	return string[0] == '\0';
    case '?':
	switch( string[0] ) {
	case '\0':
	case '/':
	    return FALSE;
	default:
	    return shellexp( string+1, pattern+1 );
	}
    case '/':
	if ( pattern[1] == '*' && pattern[2] == '*' ) {
	    char* pch = string;
	    if ( *pch != '/' ) return FALSE;                
	    if ( pattern[3] == '\0' ) return TRUE;
	    if ( pattern[3] != '/' ) {
		fprintf( stderr, "Bad expression.\n" );
		return -1;
	    }
	    while ( *pch != '\0' ) {
		if ( *pch == '/' ) {
		    if ( shellexp( pch, pattern + 3 ) ) return TRUE;
		}
		pch++;
	    }
	    return FALSE;
	}
	return string[0] == '/' && shellexp( string+1, pattern+1 );
    case '*':
	if ( string[0] == '/' ) return shellexp( string, pattern+1 );
	return shellexp( string, pattern+1 ) || 
	    (string[0] != '\0' ? shellexp( string + 1, pattern ) : FALSE );
    case '[': 
	if ( string[0] == '\0' ) return FALSE;
	{
	    int not = FALSE;
	    int okay = FALSE;
	    pattern++;
	    if ( pattern[0] == '!' || pattern[0] == '^' ) {
		not = TRUE;
		pattern++;
	    }

	    if ( pattern[0] == ']' ) {
		if ( string[0] == ']' ) { okay = TRUE; }
		pattern++;
	    }
	    
	    while( pattern[0] != ']' && pattern[0] != '\0' ) {
		if ( pattern[0] == string[0] ) {
		    okay = TRUE;
		} else if ( pattern[1] == '-' && pattern[2] != ']' ) {
		    if ( pattern[0] <= string[0] && string[0] <= pattern[2] ) {
			okay = TRUE;
		    }
		    pattern += 2;
		}
		pattern++;
	    }
	    
	    if ( pattern[0] == '\0' ) {
		fprintf( stderr, "Bad shell expression\n" );
		return -1;
	    }
	    
	    return (not ? !okay : okay) && shellexp( string + 1, pattern + 1 );
	}
    default:
	return pattern[0] == string[0] && shellexp( string + 1, pattern + 1 );
    }
}



int main(int argc, char **argv) {
    char strings[1000][1000];
    char buffer[1000];
    FILE* exp_list;
    int n_strings = 0;
    int i;
    
    for ( i = 1; i < argc; i++ ) {
	exp_list = fopen( argv[i], "r" );
	if ( exp_list == NULL ) {
	    perror( argv[i] );
	    continue;
	}
	
	while( fgets( strings[n_strings], 1000, exp_list ) ) {
	    char*pch;
	    char*buffer = strings[n_strings];
	    
	    for ( pch = buffer; *pch != '\0'; pch++ ) {
		if ( *pch == '#' ) {
		    *pch = '\0';
		    break;
		}
	    }
	    
	    for ( pch = pch - 1;
		  isspace(*pch) && pch >= buffer; 
		  pch-- )
		;
	    
	    
	    *(pch+1) = '\0';
	    
	    n_strings++; continue;
	    if ( isspace(buffer[0]) ) { 
		for ( pch = buffer + 1; isspace(*pch) && *pch; pch++ ) ;
		memmove( buffer, pch, strlen(pch) + 1 );
	    }
	    
	    if ( buffer[0] == '\0' ) continue;
	    n_strings++;
	}
	fclose( exp_list );
    } 
    
    while( fgets( buffer, 1000, stdin ) ) {
	int match;
	match = FALSE;
	
	if ( buffer[strlen(buffer)-1] == '\n' ) 
	    buffer[strlen(buffer)-1]='\0';
	
	for ( i = 0; !match && i < n_strings; i++ ) {
	    match = shellexp( buffer, strings[i] );
	}
	if ( !match ) {
	    printf( "%s\n", buffer );
	} 
    }
    return 0;
}

==> cruft_0.9.6-0.15.tar.gz: cruft-0.9.6/filter_shell.c <==
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define FALSE 0
#define TRUE  1

int shellexp(char*, char*);

int main(int argc, char **argv) {
    char strings[1000][1000];
    char buffer[1000];
    FILE* exp_list;
    int n_strings = 0;
    int i;
    
    /* Read into memory all patterns from the files specified as command line
     * arguments */
    for ( i = 1; i < argc; i++ ) {
	exp_list = fopen( argv[i], "r" );
	if ( exp_list == NULL ) {
	    perror( argv[i] );
	    continue;
	}
	
	while( fgets( strings[n_strings], 1000, exp_list ) ) {
	    char*pch;
	    char*buffer = strings[n_strings];
	    
	    for ( pch = buffer; *pch != '\0'; pch++ ) {
		/* Skip comments */
		if ( *pch == '#' ) {
		    *pch = '\0';
		    break;
		}
	    }
	    
	    /* Trim trailing whitespace */
	    for ( pch = pch - 1;
		  isspace(*pch) && pch >= buffer; 
		  pch-- )
		;
	    
	    
	    *(pch+1) = '\0';
	    
	    n_strings++; continue;
	    /* XXX: the following code is unreachable */
	    /* Trim leading whitespace */
	    if ( isspace(buffer[0]) ) { 
		for ( pch = buffer + 1; isspace(*pch) && *pch; pch++ ) ;
		memmove( buffer, pch, strlen(pch) + 1 );
	    }
	    
	    if ( buffer[0] == '\0' ) continue;
	    n_strings++;
	}
	fclose( exp_list );
    } 
    
    /* Copy lines from standard input to standard output, skipping the ones
     * which matched at least one of the loaded patterns */
    while( fgets( buffer, 1000, stdin ) ) {
	int match;
	match = FALSE;
	
	if ( buffer[strlen(buffer)-1] == '\n' ) 
	    buffer[strlen(buffer)-1]='\0';
	
	for ( i = 0; !match && i < n_strings; i++ ) {
	    match = shellexp( buffer, strings[i] );
	}
	if ( !match ) {
	    printf( "%s\n", buffer );
	} 
    }
    return 0;
}
